@@ -40,6 +40,7 @@
 #include "gegl-tile-alloc.h"
 #include "gegl-tile-backend.h"
 #include "gegl-tile-backend-swap.h"
+#include "gegl-tile-handler-empty.h"
 #include "gegl-debug.h"
 #include "gegl-buffer-config.h"
 
@@ -144,13 +145,13 @@ static gint        gegl_tile_backend_swap_get_data_cost          (ThreadParams
 static void        gegl_tile_backend_swap_free_data              (ThreadParams              *params);
 static void        gegl_tile_backend_swap_write                  (ThreadParams              *params);
 static void        gegl_tile_backend_swap_destroy                (ThreadParams              *params);
-static gpointer    gegl_tile_backend_swap_writer_thread          (gpointer ignored);       
+static gpointer    gegl_tile_backend_swap_writer_thread          (gpointer ignored);
 static GeglTile   *gegl_tile_backend_swap_entry_read             (GeglTileBackendSwap       *self,
                                                                   SwapEntry                 *entry);
 static void        gegl_tile_backend_swap_entry_write            (GeglTileBackendSwap       *self,
                                                                   SwapEntry                 *entry,
                                                                   GeglTile                  *tile);
-static SwapBlock * gegl_tile_backend_swap_block_create           (void);                   
+static SwapBlock * gegl_tile_backend_swap_block_create           (void);
 static void        gegl_tile_backend_swap_block_free             (SwapBlock                 *block);
 static SwapBlock * gegl_tile_backend_swap_block_ref              (SwapBlock                 *block,
                                                                   gint                       tile_size);
@@ -158,6 +159,7 @@ static void        gegl_tile_backend_swap_block_unref            (SwapBlock
                                                                   gint                       tile_size,
                                                                   gboolean                   lock);
 static gboolean    gegl_tile_backend_swap_block_is_unique        (SwapBlock                 *block);
+static SwapBlock * gegl_tile_backend_swap_empty_block            (void);
 static SwapEntry * gegl_tile_backend_swap_entry_create           (GeglTileBackendSwap       *self,
                                                                   gint                       x,
                                                                   gint                       y,
@@ -204,7 +206,7 @@ static gboolean    gegl_tile_backend_swap_equalfunc              (gconstpointer
                                                                   gconstpointer              b);
 static void        gegl_tile_backend_swap_constructed            (GObject                   *object);
 static void        gegl_tile_backend_swap_finalize               (GObject                   *object);
-static void        gegl_tile_backend_swap_ensure_exist           (void);                   
+static void        gegl_tile_backend_swap_ensure_exist           (void);
 static void        gegl_tile_backend_swap_class_init             (GeglTileBackendSwapClass  *klass);
 static void        gegl_tile_backend_swap_tile_cache_size_notify (GObject                   *config,
                                                                   GParamSpec                *pspec,
@@ -620,18 +622,19 @@ gegl_tile_backend_swap_write (ThreadParams *params)
       g_atomic_pointer_add (&total_uncompressed, +params->size);
     }
 
+  writing = TRUE;
+
   if (out_offset != offset)
     {
       if (lseek (out_fd, offset, SEEK_SET) < 0)
         {
           g_warning ("unable to seek to tile in buffer: %s", g_strerror (errno));
-          return;
+
+          goto error;
         }
       out_offset = offset;
     }
 
-  writing = TRUE;
-
   while (to_be_written > 0)
     {
       gint wrote;
@@ -641,7 +644,8 @@ gegl_tile_backend_swap_write (ThreadParams *params)
           g_message ("unable to write tile data to self: "
                      "%s (%d/%d bytes written)",
                      g_strerror (errno), wrote, to_be_written);
-          break;
+
+          goto error;
         }
 
       data          += wrote;
@@ -654,6 +658,17 @@ gegl_tile_backend_swap_write (ThreadParams *params)
   writing = FALSE;
 
   GEGL_NOTE (GEGL_DEBUG_TILE_BACKEND, "writer thread wrote at %i", (gint)offset);
+
+  return;
+
+error:
+  writing = FALSE;
+
+  g_atomic_pointer_add (&total_uncompressed, -params->size);
+
+  gegl_tile_backend_swap_free_block (params->block);
+
+  return;
 }
 
 static void
@@ -737,6 +752,15 @@ gegl_tile_backend_swap_entry_read (GeglTileBackendSwap *self,
   tile_size = gegl_tile_backend_get_tile_size (backend);
   bpp       = babl_format_get_bytes_per_pixel (format);
 
+  if (entry->block == gegl_tile_backend_swap_empty_block ())
+    {
+      tile = gegl_tile_handler_empty_new_tile (tile_size);
+
+      gegl_tile_mark_as_stored (tile);
+
+      return tile;
+    }
+
   g_mutex_lock (&queue_mutex);
 
   if (entry->block->link || in_progress)
@@ -799,10 +823,14 @@ gegl_tile_backend_swap_entry_read (GeglTileBackendSwap *self,
 
   g_mutex_lock (&read_mutex);
 
+  reading = TRUE;
+
   if (in_offset != offset)
     {
       if (lseek (in_fd, offset, SEEK_SET) < 0)
         {
+          reading = FALSE;
+
           g_mutex_unlock (&read_mutex);
 
           g_warning ("unable to seek to tile in buffer: %s", g_strerror (errno));
@@ -811,8 +839,6 @@ gegl_tile_backend_swap_entry_read (GeglTileBackendSwap *self,
       in_offset = offset;
     }
 
-  reading = TRUE;
-
   to_be_read = entry->block->size;
 
   while (to_be_read > 0)
@@ -1019,6 +1045,18 @@ gegl_tile_backend_swap_block_is_unique (SwapBlock *block)
   return g_atomic_int_get (&block->ref_count) == 1;
 }
 
+static SwapBlock *
+gegl_tile_backend_swap_empty_block (void)
+{
+  static SwapBlock empty_block = {
+    .ref_count = 1,
+    .link      = NULL,
+    .offset    = -1
+  };
+
+  return &empty_block;
+}
+
 static SwapEntry *
 gegl_tile_backend_swap_entry_create (GeglTileBackendSwap *self,
                                      gint                 x,
@@ -1098,41 +1136,48 @@ gegl_tile_backend_swap_set_tile (GeglTileSource *self,
 {
   GeglTileBackendSwap *swap;
   SwapEntry           *entry;
+  SwapBlock           *src_block = NULL;
+  gint                 tile_size;
+
+  swap      = GEGL_TILE_BACKEND_SWAP (self);
+  entry     = gegl_tile_backend_swap_lookup_entry (swap, x, y, z);
+  tile_size = gegl_tile_backend_get_tile_size (GEGL_TILE_BACKEND (swap));
 
   if (tile->is_zero_tile)
-    {
-      /* the tile is empty.  avoid an expensive write to disk, and just drop
-       * the existing tile (if there is one); the empty tile handler will serve
-       * a new empty tile upon request.
-       */
-      gegl_tile_backend_swap_void_tile (self, x, y, z);
-
-      gegl_tile_mark_as_stored (tile);
-
-      return GINT_TO_POINTER (TRUE);
-    }
-
-  swap  = GEGL_TILE_BACKEND_SWAP (self);
-  entry = gegl_tile_backend_swap_lookup_entry (swap, x, y, z);
+    src_block = gegl_tile_backend_swap_empty_block ();
 
   if (entry)
     {
-      if (! gegl_tile_backend_swap_block_is_unique (entry->block))
+      if (src_block)
+        {
+          if (entry->block != src_block)
+            {
+              gegl_tile_backend_swap_block_unref (
+                entry->block,
+                tile_size,
+                TRUE);
+              entry->block = gegl_tile_backend_swap_block_ref (
+                src_block,
+                tile_size);
+            }
+        }
+      else if (! gegl_tile_backend_swap_block_is_unique (entry->block))
         {
           gegl_tile_backend_swap_block_unref (
             entry->block,
-            gegl_tile_backend_get_tile_size (GEGL_TILE_BACKEND (swap)),
+            tile_size,
             TRUE);
           entry->block = gegl_tile_backend_swap_block_create ();
         }
     }
   else
     {
-      entry = gegl_tile_backend_swap_entry_create (swap, x, y, z, NULL);
-      g_hash_table_insert (swap->index, entry, entry);
+      entry = gegl_tile_backend_swap_entry_create (swap, x, y, z, src_block);
+      g_hash_table_add (swap->index, entry);
     }
 
-  gegl_tile_backend_swap_entry_write (swap, entry, tile);
+  if (! src_block)
+    gegl_tile_backend_swap_entry_write (swap, entry, tile);
 
   gegl_tile_mark_as_stored (tile);
 
@@ -1235,7 +1280,7 @@ gegl_tile_backend_swap_copy_tile (GeglTileSource           *self,
                                                        params->dst_y,
                                                        params->dst_z,
                                                        entry->block);
-      g_hash_table_insert (dst_swap->index, dst_entry, dst_entry);
+      g_hash_table_add (dst_swap->index, dst_entry);
     }
 
   return GINT_TO_POINTER (TRUE);
