@@ -11,7 +11,7 @@
  * Lesser General Public License for more details.
  *
  * You should have received a copy of the GNU Lesser General Public
- * License along with GEGL; if not, see <http://www.gnu.org/licenses/>.
+ * License along with GEGL; if not, see <https://www.gnu.org/licenses/>.
  *
  * Copyright 2006 Øyvind Kolås
  */
@@ -19,18 +19,12 @@
 
 #include "config.h"
 
-#include <string.h>
-
 #include <glib-object.h>
 
 #include "gegl.h"
-#include "gegl-types-internal.h"
 #include "gegl-operation-point-render.h"
-#include "graph/gegl-pad.h"
-#include "graph/gegl-node.h"
-#include "gegl-utils.h"
-#include "gegl-buffer-private.h"
-#include "gegl-tile-storage.h"
+#include "gegl-operation-context.h"
+#include "gegl-types-internal.h"
 
 static gboolean gegl_operation_point_render_process
                               (GeglOperation       *operation,
@@ -42,7 +36,8 @@ G_DEFINE_TYPE (GeglOperationPointRender, gegl_operation_point_render, GEGL_TYPE_
 
 static void prepare (GeglOperation *operation)
 {
-  gegl_operation_set_format (operation, "output", babl_format ("RGBA float"));
+  const Babl *format = gegl_babl_rgba_linear_float ();
+  gegl_operation_set_format (operation, "output", format);
 }
 
 static GeglNode *
@@ -63,10 +58,7 @@ gegl_operation_point_render_class_init (GeglOperationPointRenderClass *klass)
   operation_class->prepare = prepare;
 
   operation_class->detect = detect;
-  operation_class->no_cache = FALSE;
-  operation_class->get_cached_region = NULL; /* we are able to compute anything
-                                                 anywhere when we're our kind
-                                                 of class */
+  operation_class->threaded = TRUE;
 }
 
 static void
@@ -81,26 +73,37 @@ gegl_operation_point_render_process (GeglOperation       *operation,
                                      const GeglRectangle *result,
                                      gint                 level)
 {
-  GeglPad    *pad;
   const Babl *out_format;
   GeglOperationPointRenderClass *point_render_class;
 
+  GeglRectangle scaled_result = *result;
+  if (level)
+  {
+    scaled_result.x >>= level;
+    scaled_result.y >>= level;
+    scaled_result.width >>= level;
+    scaled_result.height >>= level;
+    result = &scaled_result;
+  }
+
   point_render_class = GEGL_OPERATION_POINT_RENDER_GET_CLASS (operation);
 
-  pad        = gegl_node_get_pad (operation->node, "output");
-  out_format = gegl_pad_get_format (pad);
+  out_format = gegl_operation_get_format (operation, "output");
+
   if (!out_format)
     {
-      g_warning ("%s", gegl_node_get_debug_name (operation->node));
+      g_warning ("No output format set for %s", GEGL_OPERATION_GET_CLASS (operation)->name);
+      return FALSE;
     }
-  g_assert (out_format);
 
   if ((result->width > 0) && (result->height > 0))
     {
-      GeglBufferIterator *i = gegl_buffer_iterator_new (output, result, level, out_format, GEGL_BUFFER_WRITE, GEGL_ABYSS_NONE);
+      GeglBufferIterator *i = gegl_buffer_iterator_new (output, result, level, out_format,
+                                                        GEGL_ACCESS_WRITE, GEGL_ABYSS_NONE, 2);
 
       while (gegl_buffer_iterator_next (i))
-          point_render_class->process (operation, i->data[0], i->length, &i->roi[0], level);
+          point_render_class->process (operation, i->items[0].data, i->length, &i->items[0].roi, level);
     }
+
   return TRUE;
 }
