@@ -13,8 +13,8 @@ TEST ()
   subsub = gegl_buffer_create_sub_buffer (sub, &subsubrect);
   fill (subsub, 1.0);
   print_buffer (buffer);
-  gegl_buffer_destroy (sub);
-  gegl_buffer_destroy (subsub);
-  gegl_buffer_destroy (buffer);
+  g_object_unref (sub);
+  g_object_unref (subsub);
+  g_object_unref (buffer);
   test_end ();
 }
